unit demomultidoc1;
{
   Demo of TMultiDoc component.
   
   The main form.
   
   This show how to use the TMultiDoc component and
   the function to implement to better mimic the MDI
   interface.
   
   This very simple demo can be used as a skeleton
   for a new application.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Menus, ComCtrls, MultiDoc, ChildDoc, Buttons, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Close1: TMenuItem;
    ChildControl: TPanel;  // this can also be placed in a toolbar
    BtnCloseChild: TSpeedButton;
    BtnRestoreChild: TSpeedButton;
    Cascade: TMenuItem;
    TileVer: TMenuItem;
    TileHor: TMenuItem;
    MenuItem3: TMenuItem;
    SendText: TMenuItem;
    Window1: TMenuItem;
    MultiDoc1: TMultiDoc;
    NewChild1: TMenuItem;
    Panel1: TPanel;
    procedure BtnCloseChildClick(Sender: TObject);
    procedure BtnRestoreChildClick(Sender: TObject);
    procedure CascadeClick(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure NewChild1Click(Sender: TObject);
    procedure SendTextClick(Sender: TObject);
    procedure TileHorClick(Sender: TObject);
    procedure TileVerClick(Sender: TObject);
  private
    { private declarations }
    basecaption: string;
    procedure MultiDocActiveChildChange(Sender: TObject);
    procedure MultiDocMaximize(Sender: TObject);
    procedure SetChildFocus(Sender: TObject);
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

Uses demochild;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
 // save the form caption to add the current child name later
 basecaption:=caption;
 // set the menuitem that receive the child list
 MultiDoc1.WindowList:=Window1;
 // what to do when the currnt child change
 MultiDoc1.onActiveChildChange:=@MultiDocActiveChildChange;
 // what to do when maximize state change
 MultiDoc1.onMaximize:=@MultiDocMaximize;
 // initialize the maximized control button
 ChildControl.visible:=false;
 BtnCloseChild.Glyph.LoadFromLazarusResource('CLOSE');
 BtnRestoreChild.Glyph.LoadFromLazarusResource('RESTORE');
end;

{ Close the main form }
procedure TForm1.Close1Click(Sender: TObject);
begin
  Close;
end;

{ Create a new child}
procedure TForm1.NewChild1Click(Sender: TObject);
var Child: TChildDoc;
    f : Tf_child;
begin
  // create the child container
  Child:=MultiDoc1.NewChild;
  // create the child form (stay invisible)
  f:=Tf_child.Create(Child);
  // what to do when this child want the focus
  f.onWantFocus:=@SetChildFocus;
  // assign the main panel to the child container
  Child.DockedPanel:=f.MainPanel;
  // set the child title
  Child.Caption:='demo child '+inttostr(MultiDoc1.ChildCount);
  caption:=basecaption+' - '+MultiDoc1.ActiveChild.Caption;
end;

{ show how to access to the current child objects from the main form }
procedure TForm1.SendTextClick(Sender: TObject);
begin
 if MultiDoc1.ActiveObject is Tf_child then
   with MultiDoc1.ActiveObject as Tf_child do
      memo1.lines.Add('Text from main form');
end;

{ Restore child size }
procedure TForm1.BtnRestoreChildClick(Sender: TObject);
begin
  MultiDoc1.Maximized:=not MultiDoc1.Maximized;
  // or MultiDoc1.Maximized:=false;
end;

{ Close the current child }
procedure TForm1.BtnCloseChildClick(Sender: TObject);
begin
if MultiDoc1.ActiveObject is Tf_child then
   MultiDoc1.ActiveChild.close;
end;

{ cascade the child windows }
procedure TForm1.CascadeClick(Sender: TObject);
begin
  MultiDoc1.Cascade;
end;

{ tile the child windows horizontally}
procedure TForm1.TileHorClick(Sender: TObject);
begin
  MultiDoc1.TileHorizontal;
end;

{ tile the child windows vertically}
procedure TForm1.TileVerClick(Sender: TObject);
begin
  MultiDoc1.TileVertical;
end;

{ when the active child change, adjust the main form caption }
procedure TForm1.MultiDocActiveChildChange(Sender: TObject);
begin
if MultiDoc1.ActiveChild<>nil then
   caption:=basecaption+' - '+MultiDoc1.ActiveChild.Caption
else
   caption:=basecaption;
end;

{ give focus to a child that request that }
procedure TForm1.SetChildFocus(Sender: TObject);
var i:integer;
begin
for i:=0 to MultiDoc1.ChildCount-1 do
   if MultiDoc1.Childs[i].DockedObject=Sender then
      MultiDoc1.setActiveChild(i);

end;

{ when the child maximize, show the close,restore button }
procedure TForm1.MultiDocMaximize(Sender: TObject);
begin
 ChildControl.visible:=MultiDoc1.Maximized;
end;


initialization
  {$I demomultidoc1.lrs}

end.

